from __future__ import annotations

import shutil
from pathlib import Path
from subprocess import CalledProcessError
from zipfile import ZipFile

import pytest

from wheel.wheelfile import WheelFile

from .util import run_command

TESTWHEEL_NAME = "test-1.0-py2.py3-none-any.whl"
TESTWHEEL_PATH = Path(__file__).parent.parent / "testdata" / TESTWHEEL_NAME


@pytest.fixture
def wheelpath(tmp_path: Path) -> Path:
    wheels_dir = tmp_path / "wheels"
    wheels_dir.mkdir()
    fn = wheels_dir / TESTWHEEL_NAME
    shutil.copy(TESTWHEEL_PATH, fn)
    return fn


def test_tags_no_args(wheelpath: Path) -> None:
    newname = run_command("tags", wheelpath).strip()
    assert newname == TESTWHEEL_NAME
    assert wheelpath.exists()


def test_python_tags(wheelpath: Path) -> None:
    newname = run_command("tags", "--python-tag", "py3", wheelpath).strip()
    assert newname == TESTWHEEL_NAME.replace("py2.py3", "py3")
    output_file = wheelpath.parent / newname
    with WheelFile(output_file) as f:
        output = f.read(f.dist_info_path + "/WHEEL")

    assert (
        output == b"Wheel-Version: 1.0\nGenerator: bdist_wheel (0.30.0)"
        b"\nRoot-Is-Purelib: false\nTag: py3-none-any\n\n"
    )
    output_file.unlink()

    newname = run_command("tags", wheelpath, "--python-tag", "py2.py3").strip()
    assert newname == TESTWHEEL_NAME

    newname = run_command("tags", "--remove", "--python-tag", "+py4", wheelpath).strip()
    assert not wheelpath.exists()
    assert newname == TESTWHEEL_NAME.replace("py2.py3", "py2.py3.py4")
    output_file = wheelpath.parent / newname
    output_file.unlink()


def test_abi_tags(wheelpath: Path) -> None:
    newname = run_command("tags", wheelpath, "--abi-tag", "cp33m").strip()
    assert newname == TESTWHEEL_NAME.replace("none", "cp33m")
    output_file = wheelpath.parent / newname
    output_file.unlink()

    newname = run_command("tags", wheelpath, "--abi-tag", "cp33m.abi3").strip()
    assert newname == TESTWHEEL_NAME.replace("none", "abi3.cp33m")
    output_file = wheelpath.parent / newname
    output_file.unlink()

    newname = run_command("tags", wheelpath, "--abi-tag", "none").strip()
    assert newname == TESTWHEEL_NAME

    newname = run_command(
        "tags", wheelpath, "--remove", "--abi-tag", "+abi3.cp33m"
    ).strip()
    assert not wheelpath.exists()
    assert newname == TESTWHEEL_NAME.replace("none", "abi3.cp33m.none")
    output_file = wheelpath.parent / newname
    output_file.unlink()


def test_plat_tags(wheelpath: Path) -> None:
    newname = run_command("tags", "--platform-tag", "linux_x86_64", wheelpath).strip()
    assert newname == TESTWHEEL_NAME.replace("any", "linux_x86_64")
    output_file = wheelpath.parent / newname
    assert output_file.exists()
    output_file.unlink()

    newname = run_command(
        "tags", "--platform-tag", "linux_x86_64.win32", wheelpath
    ).strip()
    assert newname == TESTWHEEL_NAME.replace("any", "linux_x86_64.win32")
    output_file = wheelpath.parent / newname
    assert output_file.exists()
    output_file.unlink()

    newname = run_command(
        "tags", "--platform-tag", "+linux_x86_64.win32", wheelpath
    ).strip()
    assert newname == TESTWHEEL_NAME.replace("any", "any.linux_x86_64.win32")
    output_file = wheelpath.parent / newname
    assert output_file.exists()
    output_file.unlink()

    newname = run_command(
        "tags", "--platform-tag", "+linux_x86_64.win32", wheelpath
    ).strip()
    assert newname == TESTWHEEL_NAME.replace("any", "any.linux_x86_64.win32")
    output_file = wheelpath.parent / newname
    assert output_file.exists()

    newname2 = run_command("tags", "--platform-tag=-any", output_file).strip()
    output_file.unlink()

    assert newname2 == TESTWHEEL_NAME.replace("any", "linux_x86_64.win32")
    output_file2 = wheelpath.parent / newname2
    assert output_file2.exists()
    output_file2.unlink()

    newname = run_command("tags", "--platform-tag", "any", wheelpath).strip()
    assert newname == TESTWHEEL_NAME


def test_build_tag(wheelpath: Path) -> None:
    newname = run_command("tags", "--build", "1bah", wheelpath).strip()
    assert newname == TESTWHEEL_NAME.replace("-py2", "-1bah-py2")
    output_file = wheelpath.parent / newname
    assert output_file.exists()

    newname = run_command("tags", "--build", "", wheelpath).strip()
    assert newname == TESTWHEEL_NAME
    output_file.unlink()


@pytest.mark.parametrize(
    "build_tag, error",
    [
        pytest.param("foo", "build tag must begin with a digit", id="digitstart"),
        pytest.param("1-f", "invalid character ('-') in build tag", id="hyphen"),
    ],
)
def test_invalid_build_tag(wheelpath: Path, build_tag: str, error: str) -> None:
    with pytest.raises(CalledProcessError) as exc_info:
        run_command("tags", "--build", build_tag, wheelpath, catch_systemexit=False)

    exc = exc_info.value
    assert exc.returncode == 2
    assert f"error: argument --build: {error}" in exc.stderr


def test_multi_tags(wheelpath: Path) -> None:
    newname = run_command(
        "tags",
        "--platform-tag",
        "linux_x86_64",
        "--python-tag",
        "+py4",
        "--build",
        "1",
        wheelpath,
    ).strip()
    assert newname == "test-1.0-1-py2.py3.py4-none-linux_x86_64.whl"

    output_file = wheelpath.parent / newname
    assert output_file.exists()
    with WheelFile(output_file) as f:
        output = f.read(f.dist_info_path + "/WHEEL")

    assert output == (
        b"Wheel-Version: 1.0\n"
        b"Generator: bdist_wheel (0.30.0)\nRoot-Is-Purelib: false\n"
        b"Tag: py2-none-linux_x86_64\n"
        b"Tag: py3-none-linux_x86_64\n"
        b"Tag: py4-none-linux_x86_64\n"
        b"Build: 1\n\n"
    )
    output_file.unlink()


def test_tags_command(wheelpath: Path) -> None:
    newname = run_command(
        "tags",
        "--python-tag",
        "py3",
        "--abi-tag",
        "cp33m",
        "--platform-tag",
        "linux_x86_64",
        "--build",
        "7",
        wheelpath,
    ).strip()
    assert "test-1.0-7-py3-cp33m-linux_x86_64.whl" == newname
    output_file = wheelpath.parent / newname
    output_file.unlink()


def test_tags_command_del(wheelpath: Path) -> None:
    newname = run_command(
        "tags",
        "--python-tag",
        "+py4",
        "--abi-tag",
        "cp33m",
        "--platform-tag",
        "linux_x86_64",
        "--remove",
        wheelpath,
    ).strip()
    assert "test-1.0-py2.py3.py4-cp33m-linux_x86_64.whl" == newname
    output_file = wheelpath.parent / newname
    output_file.unlink()


def test_permission_bits(wheelpath: Path) -> None:
    newname = run_command("tags", "--python-tag", "+py4", wheelpath).strip()
    assert "test-1.0-py2.py3.py4-none-any.whl" == newname
    output_file = wheelpath.parent / newname

    with ZipFile(output_file, "r") as outf, ZipFile(wheelpath, "r") as inf:
        for member in inf.namelist():
            member_info = inf.getinfo(member)
            if member_info.is_dir():
                continue

            if member_info.filename.endswith("/RECORD"):
                continue

            out_attr = outf.getinfo(member).external_attr
            inf_attr = member_info.external_attr
            assert out_attr == inf_attr, (
                f"{member} 0x{out_attr:012o} != 0x{inf_attr:012o}"
            )

    output_file.unlink()



from __future__ import annotations

import email.policy
import os
from email.message import Message
from email.parser import BytesParser
from zipfile import Path, ZipFile

import pytest
from pytest import TempPathFactory

from .util import run_command

THISDIR = os.path.dirname(__file__)
TESTWHEEL_NAME = "test-1.0-py2.py3-none-any.whl"
TESTWHEEL_PATH = os.path.join(THISDIR, "..", "testdata", TESTWHEEL_NAME)


@pytest.mark.filterwarnings("error:Duplicate name")
@pytest.mark.parametrize(
    "build_tag_arg, existing_build_tag, filename",
    [
        pytest.param(None, None, "test-1.0-py2.py3-none-any.whl", id="nobuildnum"),
        pytest.param("2b", None, "test-1.0-2b-py2.py3-none-any.whl", id="newbuildarg"),
        pytest.param(None, "3", "test-1.0-3-py2.py3-none-any.whl", id="oldbuildnum"),
        pytest.param("", "3", "test-1.0-py2.py3-none-any.whl", id="erasebuildnum"),
    ],
)
def test_pack(
    tmp_path_factory: TempPathFactory,
    tmp_path: Path,
    build_tag_arg: str | None,
    existing_build_tag: str | None,
    filename: str,
) -> None:
    unpack_dir = tmp_path_factory.mktemp("wheeldir")
    with ZipFile(TESTWHEEL_PATH) as zf:
        old_record = zf.read("test-1.0.dist-info/RECORD")
        old_record_lines = sorted(
            line.rstrip()
            for line in old_record.split(b"\n")
            if line and not line.startswith(b"test-1.0.dist-info/WHEEL,")
        )
        zf.extractall(unpack_dir)

    if existing_build_tag:
        # Add the build number to WHEEL
        wheel_file_path = unpack_dir.joinpath("test-1.0.dist-info").joinpath("WHEEL")
        wheel_file_content = wheel_file_path.read_bytes()
        assert b"Build" not in wheel_file_content
        wheel_file_content += b"Build: 3\r\n"
        wheel_file_path.write_bytes(wheel_file_content)

    args = ["--dest", tmp_path, unpack_dir]
    if build_tag_arg is not None:
        (args.insert(3, "--build"),)
        args.insert(4, build_tag_arg)

    run_command("pack", *args)
    new_wheel_path = tmp_path.joinpath(filename)
    assert new_wheel_path.is_file()

    with ZipFile(new_wheel_path) as zf:
        new_record = zf.read("test-1.0.dist-info/RECORD")
        new_record_lines = sorted(
            line.rstrip()
            for line in new_record.split(b"\n")
            if line and not line.startswith(b"test-1.0.dist-info/WHEEL,")
        )

        parser = BytesParser(policy=email.policy.compat32)
        new_wheel_file_content = parser.parsebytes(zf.read("test-1.0.dist-info/WHEEL"))

    assert new_record_lines == old_record_lines

    # Line endings and trailing blank line will depend on whether WHEEL
    # was modified.  Circumvent this by comparing parsed key/value pairs.
    expected_wheel_content = Message()
    expected_wheel_content["Wheel-Version"] = "1.0"
    expected_wheel_content["Generator"] = "bdist_wheel (0.30.0)"
    expected_wheel_content["Root-Is-Purelib"] = "false"
    expected_wheel_content["Tag"] = "py2-none-any"
    expected_wheel_content["Tag"] = "py3-none-any"
    expected_build_num = (
        build_tag_arg if build_tag_arg is not None else existing_build_tag
    )
    if expected_build_num:
        expected_wheel_content["Build"] = expected_build_num

    assert sorted(new_wheel_file_content.items()) == sorted(
        expected_wheel_content.items()
    )



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wheel', 'wheel._commands']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['packaging >= 24.0']

entry_points = \
{'console_scripts': ['wheel = wheel._commands:main'],
 'distutils.commands': ['bdist_wheel = wheel.bdist_wheel:bdist_wheel']}

setup(name='wheel',
      version='0.47.0',
      description='Command line tool for manipulating wheel files',
      author=None,
      author_email='Daniel Holth <dholth@fastmail.fm>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.9',
     )
